/*
 * Copyright (C) 2013 Canonical, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import Ubuntu.Application 0.1
import ApplicationLauncher 0.1 //executes apps only
import "../Applications/applications.js" as ApplicationsModel

Item {
    id: applicationManager

    property var mainStageApplications: ApplicationManager.mainStageApplications
    property var sideStageApplications: ApplicationManager.sideStageApplications
    property var mainStageFocusedApplication: ApplicationManager.mainStageFocusedApplication
    property var sideStageFocusedApplication: ApplicationManager.sideStageFocusedApplication
    property bool sideStageEnabled: true
    signal focusRequested(string desktopFile)
    property bool keyboardVisible: ApplicationManager.keyboardVisible
    property int keyboardHeight: ApplicationManager.keyboardHeight

    Component.onCompleted: {
        // Start the watcher so that the ApplicationManager applications model can be populated.
        ApplicationManager.startWatcher();
    }

    function activateApplication(desktopFile, argument) {
        var application;
        application = getApplicationFromDesktopFile(desktopFile, ApplicationsModel.MainStage);
        if (application == null) {
            application = getApplicationFromDesktopFile(desktopFile, ApplicationsModel.SideStage);
        }
        if (application != null) {
            return application;
        }

        var app = ApplicationsModel.createApplicationForDesktopFile(desktopFile);
        if(!app) {
            console.log("ERROR: Application '" + desktopFile + "' not found, unable to activate");
            return null;
        }

        if (!app.exec || !app.desktopFile) {
            console.log("ERROR: Application '" + desktopFile + "' has no exec or desktop file");
            return null;
        }

        if (argument == undefined) {
            argument = "";
        }

        var sideStage = applicationManager.sideStageEnabled
                        && (app.stage == ApplicationsModel.SideStage);
        var stageHint = sideStage ? "--stage_hint=side_stage" : ""

        // Application not running, so execute it
        var command = "%1 %2 --desktop_file_hint=%3 %4".arg(app.exec).arg(argument).arg(app.desktopFile).arg(stageHint);
        if (applicationLauncher.execute(command)) {
            /* FIXME: application is null unfortunately; return app instead that
               has a similar API. This will be fixed by using qtubuntu's
               ApplicationManager.startProcess().
            */
            return app;
        } else {
            console.log("ERROR: Failed to launch", app.exec);
            return null;
        }
    }

    function focusApplication(application) {
        if (application == null || application == undefined) {
            return;
        }

        ApplicationManager.focusApplication(application.handle);
    }

    function unfocusCurrentApplication() {
        ApplicationManager.unfocusCurrentApplication(ApplicationsModel.MainStage);
    }

    function moveRunningApplicationStackPosition(from, to, stage) {
        var applications;
        if (stage == ApplicationsModel.SideStage) {
            applications = ApplicationManager.sideStageApplications;
        } else {
            applications = ApplicationManager.mainStageApplications;
        }
        // FIXME: applications.move(0, 0) crashes
        if (from !== to && from >= 0 && to >= 0) applications.move(from, to);
    }

    function getApplicationFromDesktopFile(desktopFile, stage) {
        var applications;
        if (stage == ApplicationsModel.SideStage) {
            applications = ApplicationManager.sideStageApplications;
        } else {
            applications = ApplicationManager.mainStageApplications;
        }

        for (var i = 0; i < applications.count; i++ ) {
            var application = applications.get(i);
            if (application.desktopFile == desktopFile) {
                return application;
            }
        }
        return null;
    }

    Connections {
        target: ApplicationManager
        onFocusRequested: {
            // FIXME: hardcoded mapping from ApplicationManager.FavoriteApplications
            // enum to desktop files
            var desktopFile
            if (favoriteApplication == ApplicationManager.CameraApplication) {
                desktopFile = "/usr/share/applications/camera-app.desktop"
            } else if (favoriteApplication == ApplicationManager.GalleryApplication) {
                desktopFile = "/usr/share/applications/gallery-app.desktop"
            } else if (favoriteApplication == ApplicationManager.ShareApplication) {
                desktopFile = "/usr/share/applications/share-app.desktop"
            } else if (favoriteApplication == ApplicationManager.BrowserApplication) {
                desktopFile = "/usr/share/applications/webbrowser-app.desktop"
            } else if (favoriteApplication == 4) { // ApplicationManager.PhoneApplication
                desktopFile = "/usr/share/applications/phone-app.desktop"
            }
            applicationManager.focusRequested(desktopFile);
        }
    }
}
