/*
 * Copyright (C) 2013 Canonical, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Utils 0.1
import "../Components"
import "../Components/ListItems"
import "Apps"

LensView {
    id: root

    // FIXME: a way to aggregate these models would be ideal
    property var mainStageApplicationsModel: shell.applicationManager.mainStageApplications
    property var sideStageApplicationModel: shell.applicationManager.sideStageApplications

    onMovementStarted: listView.showHeader()
    onPositionedAtBeginning: {
        if (isCurrent) {
            listView.positionAtBeginning()
        }
    }

    ListModel {
        id: categoryListModel
        // specifies app categories, type of delegate and model used in each category
        ListElement { category: "Running apps";           component: "RunningApplicationsGrid"; modelName: "RunningApplicationsModel" }
        ListElement { category: "Frequently used";        component: "ApplicationsFilterGrid";  modelName: "FrequentlyUsedAppsModel" }
        ListElement { category: "Installed";              component: "ApplicationsFilterGrid";  modelName: "InstalledApplicationsModel" }
        ListElement { category: "Available for download"; component: "ApplicationsFilterGrid";  modelName: "AppsAvailableForDownloadModel" }
    }

    InstalledApplicationsModel { id: installedApplicationsModel }
    FrequentlyUsedAppsModel { id: frequentlyUsedAppsModel }
    AppsAvailableForDownloadModel { id: appsAvailableForDownloadModel }

    property var categoryModels: {
        "FrequentlyUsedAppsModel": frequentlyUsedAppsModel,
        "InstalledApplicationsModel": installedApplicationsModel,
        "AppsAvailableForDownloadModel": appsAvailableForDownloadModel,
    }

    /* Workaround for bug: https://bugreports.qt-project.org/browse/QTBUG-28403
       When using Loader to load external QML file in the list deelgate, the ListView has
       a bug where it can position the delegate content to overlap the section header
       of the ListView - a workaround is to use sourceComponent of Loader instead */
    Component { id: runningApplicationsGrid; RunningApplicationsGrid {} }
    Component { id: applicationsFilterGrid; ApplicationsFilterGrid {} }
    property var componentModels: {
        "RunningApplicationsGrid": runningApplicationsGrid,
        "ApplicationsFilterGrid": applicationsFilterGrid,
    }

    ListViewWithPageHeader {
        id: listView
        anchors.fill: parent
        model: SortFilterProxyModel {
            model: categoryListModel
            filterRole: 0 // 0 == modelName
            filterRegExp: (mainStageApplicationsModel.count === 0
                           && sideStageApplicationModel.count === 0) ? RegExp("RunningApplicationsModel") : RegExp("")
            invertMatch: true
        }

        onAtYEndChanged: if (atYEnd) endReached()
        onMovingChanged: if (moving && atYEnd) endReached()

        delegate: Base {
            id: container
            highlightWhenPressed: false

            Loader {
                anchors {
                    top: parent.top
                    left: parent.left
                    right: parent.right
                }
                sourceComponent: componentModels[component]
                onLoaded: {
                    if (modelName == "RunningApplicationsModel") {
                        item.firstModel = mainStageApplicationsModel
                        item.secondModel = sideStageApplicationModel
                    } else {
                        item.model = categoryModels[modelName]
                    }
                }
                asynchronous: true
            }
        }

        sectionProperty: "category"
        sectionDelegate: Header {
            width: listView.width
            text: section
        }
        pageHeader: PageHeader {
            width: listView.width
            text: "Apps"
        }
    }
}
