/*
 * Copyright (C) 2013 Canonical, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Unity 0.1

Item {
    id: dashBar

    property var model
    property int currentIndex

    property int lineHeight: units.dp(2)
    property int itemSize: units.gu(7)
    property int iconSize: units.gu(2.5)
    property int initialHeight: lineHeight
    property int maxHeight: units.gu(6)

    width: units.gu(40)
    height: initialHeight
    Behavior on height {NumberAnimation { duration: 300; easing.type: Easing.OutQuad}}

    state: "initial"

    signal itemSelected(int index)

    function startNavigation() {
        timeout.stop()
        dashBar.state = "navigation"
    }

    function stopNavigation() {
        timeout.restart()
    }

    function finishNavigation() {
        dashBar.state = "initial"
    }

    Rectangle {
        color: "black"
        anchors.fill: parent

        ListView {
            id: row
            model: dashBar.model
            orientation: ListView.Horizontal
            width: Math.min(Math.max(dashBar.width/2, units.gu(40)), count * itemSize)
            height: parent.height
            anchors.horizontalCenter: parent.horizontalCenter
            currentIndex: dashBar.currentIndex
            onMovingChanged: if (moving) { timeout.stop() } else { timeout.restart() }
            interactive: visibleArea.widthRatio < 1 && dashBar.state == "navigation"
            delegate:
                Item {
                    width: itemSize
                    height: maxHeight
                    anchors.top: parent.top

                    Component.onCompleted: z = 0

                    Image {
                        anchors.centerIn: parent
                        /* FIXME: should be 'source: lens.iconHint' but the
                           lenses do not contain the right icons yet
                        */
                        source: "graphics/lensIcons/%1.png".arg(lens.name)
                        height: iconSize
                        width: iconSize
                        opacity: index == currentIndex ? 1 : 1 //same opacity for now
                        smooth: true
                    }
                    MouseArea {
                        enabled: dashBar.state == "navigation"
                        anchors.fill: parent
                        onPressed: {
                            timeout.stop()
                            if (dashbar.state == "initial") {
                                dashbar.state = "navigation"
                                mouse.accepted = false
                            }
                        }

                        onReleased: {
                            timeout.restart()
                        }

                        onClicked: {
                            dashBar.itemSelected(index)
                        }
                    }
                }
            highlightFollowsCurrentItem: false
            highlight:
                Rectangle {
                    color: "#dd4814"
                    height: lineHeight
                    width: row.currentItem.width
                    anchors.bottom: parent.bottom
                    x: row.currentItem.x

                    Component.onCompleted: z = 1

                    Behavior on x {NumberAnimation { duration: 150; easing.type: Easing.OutCubic}}
                }
        }
    }

    Timer {
        id: timeout
        interval: 1500
        running: false
        repeat: false
        onTriggered: finishNavigation()
    }

    states: [
        State {
            name: "initial"
            PropertyChanges {
                target: dashBar
                height: initialHeight
            }
        },
        State {
            name: "navigation"
            PropertyChanges {
                target: dashBar
                height: maxHeight
            }
        }
    ]
}

