/*
 * Copyright (C) 2013 Canonical, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import QtQuick.XmlListModel 2.0
import "../Components"

Showable {
    id: greeter
    enabled: shown
    created: greeterContentLoader.status == Loader.Ready && greeterContentLoader.item.ready

    property alias model: userList
    property bool locked: shown && multiUser && !greeterContentLoader.guestAccount

    readonly property bool narrowMode: width <= units.gu(60)
    readonly property bool multiUser: !narrowMode // TODO: populate with real value
    
    signal selected(int uid)
    signal unlocked(int uid)
    
    ListModel {
        id: userList
        ListElement { name: "Guest"; background: "graphics/tablet_background.jpg" }
    }

    // TODO: Replace with a ModelAggregator once we have one.
    // Check Unity2D for a possibly usable one.
    XmlListModel {
        id: xmlReader
        source: "/usr/share/demo-assets/shell/users.xml"
        query: "/users/user"

        XmlRole { name: "name"; query: "username/string()" }
        XmlRole { name: "password"; query: "password/string()" }
        XmlRole { name: "background"; query: "background/string()" }
        XmlRole { name: "infographic"; query: "infographic/string()" }

        onStatusChanged: {
            if (status == XmlListModel.Ready) {
                // Need to reset the model to make sure everything updates correctly.
                // Otherwise currentIndex, count etc could stay the same and bindings are not updated.
                greeterContentLoader.currentIndex = -1;
                greeterContentLoader.model = undefined;

                userList.clear();
                for (var i = 0; i < xmlReader.count; i++) {
                    userList.append(xmlReader.get(i));
                }
                userList.append({ "name": "Guest", background: "graphics/tablet_background.jpg" });

                greeterContentLoader.model = userList;
                greeterContentLoader.currentIndex = 0;
            }
        }
    }

    Loader {
        id: greeterContentLoader
        objectName: "greeterContentLoader"
        anchors.fill: parent
        property variant model: userList
        property int currentIndex: 0
        property bool guestAccount: item ? item.guestAccount : false

        source: required ? "GreeterContent.qml" : ""
        

        Connections {
            target: greeterContentLoader.item

            onSelected: greeter.selected(uid);
            onUnlocked: greeter.unlocked(uid);
        }
    }
}
