# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""qml-phone-shell autopilot tests."""

import os.path

from autopilot.introspection.qt import QtIntrospectionTestMixin
from autopilot.testcase import AutopilotTestCase
from autopilot.matchers import Eventually
from testtools.matchers import Equals

from qml_phone_shell.emulators.main_window import MainWindow

class FormFactors(object):
    Phone, Tablet, Desktop = range(3)

class ShellTestCase(AutopilotTestCase, QtIntrospectionTestMixin):

    """A common test case class that provides several useful methods for shell tests."""

    def setUp(self, geometry, grid_size):
        super(ShellTestCase, self).setUp()
        # Lets assume we are installed system wide if this file is somewhere in /usr
        os.environ['GRID_UNIT_PX'] = grid_size
        self.grid_size = int(grid_size)
        if os.path.realpath(__file__).startswith("/usr/"):
            self.launch_test_installed(geometry)
        else:
            self.launch_test_local(geometry)

    def launch_test_local(self, geometry):
        self.app = self.launch_test_application(
            "../../qml-phone-shell", "-geometry", geometry, "-frameless")

    def launch_test_installed(self, geometry):
        if self.running_on_device():
            self.app = self.launch_test_application(
               "qml-phone-shell", "-fullscreen")
        else:
            self.app = self.launch_test_application(
               "qml-phone-shell", "-geometry", geometry, "-frameless")

    def skipWrapper(*args, **kwargs):
        pass

    def form_factor(self):
        return FormFactors.Desktop

    def __getattribute__(self, attr_name):
        attr = object.__getattribute__(self, attr_name);
        if attr_name.startswith("test_"):
            try:
                if self.form_factor() in attr.blacklist:
                    return self.skipWrapper
            except:
                pass
        return attr


    @staticmethod
    def running_on_device():
        return os.path.isfile('/system/usr/idc/autopilot-finger.idc')

    @property
    def main_window(self):
        return MainWindow(self.app)

