# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for the Shell"""

from __future__ import absolute_import

from testtools.matchers import Equals
from autopilot.matchers import Eventually
from autopilot.emulators.X11 import ScreenGeometry

from qml_phone_shell.tests import ShellTestCase, FormFactors
from qml_phone_shell.tests.testcases import TestShell

class TestNexus4(TestShell, ShellTestCase):

    def setUp(self):
        sg = ScreenGeometry()
        if (sg.get_screen_width() >= 768 and sg.get_screen_height() >= 1280):
            super(TestNexus4, self).setUp("768x1280", "18")
        else:
            super(TestNexus4, self).setUp("384x640", "9")
        dash = self.main_window.get_dash()
        self.assertThat(dash.showLensOnLoaded, Eventually(Equals(""), timeout=30))

    def tearDown(self):
        super(TestNexus4, self).tearDown()

    def form_factor(self):
        return FormFactors.Phone



class TestNexus7(TestShell, ShellTestCase):

    def setUp(self):
        sg = ScreenGeometry()
        if (sg.get_screen_width() >= 800 and sg.get_screen_height() >= 1280):
            super(TestNexus7, self).setUp("800x1280", "14")
        else:
            super(TestNexus7, self).setUp("400x640", "7")
        dash = self.main_window.get_dash()
        self.assertThat(dash.showLensOnLoaded, Eventually(Equals(""), timeout=30))

    def tearDown(self):
        super(TestNexus7, self).tearDown()

    def form_factor(self):
        return FormFactors.Tablet


class TestNexus10(TestShell, ShellTestCase):

    def setUp(self):
        sg = ScreenGeometry()
        if (sg.get_screen_width() >= 2560 and sg.get_screen_height() >= 1600):
            super(TestNexus10, self).setUp("2560x1600", "20")
        elif (sg.get_screen_width() >= 1280 and sg.get_screen_height() >= 800):
            super(TestNexus10, self).setUp("1280x800", "10")
        else:
            super(TestNexus10, self).setUp("640x400", "5")
        dash = self.main_window.get_dash()
        self.assertThat(dash.showLensOnLoaded, Eventually(Equals(""), timeout=45))

    def tearDown(self):
        super(TestNexus10, self).tearDown()

    def form_factor(self):
        return FormFactors.Tablet
